/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package bresenhamapplet;

import java.awt.Graphics;

/**
 * This class contains the algorithm for rasterizing an ellipse.
 * @author SUN2K
 */
public class EllipseAlgorithms {

    /**
     * Plots a circle point to graphics object, zoomed.
     * @param g Graphics object used for actual pixel plotting
     * @param x x-coordinate of pixel
     * @param y y-coordinate of pixel
     * @param size size of point to draw; use 1 for normal plotting
     */
    private static void setPixel(Graphics g, int x, int y, int size)
    {
        g.fillRect(x, y, size, size);
    }

    /**
     * Helper function to draw 4 symmteric points of an ellipse.
     * @param g Graphics object used for actual pixel plotting
     * @param mx x-coordinate of center of ellipse
     * @param my y-coordinate of center of ellipse
     * @param x x-value for point on ellipse from center
     * @param y y-value for point on ellipse from center
     * @param multiplier zoom multiplier, use 1 for normal drawing
     */
    private static void drawSymmetricPoints(Graphics g, int mx, int my, int x, int y, int multiplier)
    {
        setPixel(g, (mx + x) * multiplier, (my + y) * multiplier, multiplier);
        setPixel(g, (mx - x) * multiplier, (my + y) * multiplier, multiplier);
        setPixel(g, (mx - x) * multiplier, (my - y) * multiplier, multiplier);
        setPixel(g, (mx + x) * multiplier, (my - y) * multiplier, multiplier);
    }


    
    /**
     * Bresenham ellipse rasterising algorithm, using integer numbers.
     * @param g Graphics object where ellipse is plotted
     * @param mx x-coordinate of center of ellipse
     * @param my y-coordinate of center of ellipse
     * @param radiusX radius in x-direction of ellipse
     * @param radiusY radius in y-direction of ellipse
     * @param multiplier zoom multiplier, use 1 for normal drawing
     */
    public static void drawEllipseBresenham(Graphics g, int mx, int my, int radiusX, int radiusY, int multiplier)
    {
        int x, y, changeX, changeY, e, twoASquare, twoBSquare, stoppingX, stoppingY;
        
        mx /= multiplier;
        my /= multiplier;
        radiusX /= multiplier;
        radiusY /= multiplier;

        twoASquare = 2 * radiusX * radiusX;
        twoBSquare = 2 * radiusY * radiusY;
        x = radiusX;
        y = 0;
        changeX = radiusY * radiusY * (1 - 2 * radiusX);
        changeY = radiusX * radiusX;
        e = 0;
        stoppingX = twoBSquare * radiusX;
        stoppingY = 0;

        while (stoppingX >= stoppingY)
        {
            drawSymmetricPoints(g, mx, my, x, y, multiplier);
            y++;
            stoppingY += twoASquare;
            e += changeY;
            changeY += twoASquare;
            if ( (2*e + changeX) > 0)
            {
                x--;
                stoppingX -= twoBSquare;
                e += changeX;
                changeX += twoBSquare;
            }
        }

        x = 0;
        y = radiusY;
        changeX = radiusY * radiusY;
        changeY = radiusX * radiusX * (1 - 2 * radiusY);
        e = 0;
        stoppingX = 0;
        stoppingY = twoASquare * radiusY;
        while (stoppingX <= stoppingY)
        {
            drawSymmetricPoints(g, mx, my, x, y, multiplier);
            x++;
            stoppingX += twoBSquare;
            e += changeX;
            changeX += twoBSquare;
            if ( (2*e + changeY) > 0)
            {
                y--;
                stoppingY -= twoASquare;
                e += changeY;
                changeY += twoASquare;
            }
        }
    }
}
