/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package bresenhamapplet;

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;


/**
 *
 * @author Sunshine2k
 */
public class MovablePoint extends javax.swing.JPanel {

    private static final int SIZE = 8;
    private static final int SIZE2 = SIZE / 2;
    private static BresenhamPanel parent;

    private int oldX = 0, oldY = 0;

    private Vector<BresenhamEventListener> BresenhamEventListeners = new Vector<BresenhamEventListener>();
    
    /**
     * Main constructor.
     * @param parentPanel the panel where this point belongs to.
     * @param x x-coordinate of point position
     * @param y y-coordinate of point position
     */
    public MovablePoint(BresenhamPanel parentPanel, int x, int y)
    {
        parent = parentPanel;
        this.setLocation(x, y);
        this.setSize(SIZE, SIZE);
        this.setBackground(Color.BLACK);
        parent.add(this);
        repaint();

        setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));

        this.addMouseMotionListener(new MouseAdapter()
        {
            @Override
            public void mouseMoved(MouseEvent event)
            {

               oldX = event.getX();
               oldY = event.getY();
            }

            @Override
            public void mouseDragged(MouseEvent event)
            {
                // prevent draggin outside the applet
                if (getLocation().x + event.getX() > getParentBounds().width ||
                    getLocation().x + event.getX() < 0 ||
                    getLocation().y + event.getY() > getParentBounds().height ||
                    getLocation().y + event.getY() < 0)
                {
                    return;
                }

                int xx = getLocation().x;
                int yy = getLocation().y;
                setLocation(xx +  event.getX() - oldX, yy + event.getY() - oldY);
                informListener(getMidX(), getMidY());
                repaintParent();
            }
        });
    }

    /**
     * Repaint parent panel.
     */
    private void repaintParent()
    {
        parent.repaint();
    }

    /**
     * Helper function to get size of parent panel.
     * @return rectangle of parent panel.
     */
    public Rectangle getParentBounds()
    {
        return parent.getBounds();
    }

    @Override
    public int getX() { return (getLocation().x); }
    @Override
    public int getY() { return (getLocation().y); }

    /**
     * Get position of this point.
     * @return x-coordinate of actual center of this point
     */
    public int getMidX()
    {
        return getLocation().x + SIZE2;
    }

    /**
     * Get position of this point.
     * @return y-coordinate of actual center of this point
     */
    public int getMidY()
    {
        return getLocation().y + SIZE2;
    }

    /**
     * Register to mouse events of this point to get notified if this point is dragged
     * @param bel The interface that should get be notified & called.
     */
    public void addMovablePointListener(BresenhamEventListener bel)
    {
        if (BresenhamEventListeners.contains(bel))
            return;
        BresenhamEventListeners.addElement(bel);
    }

    /**
     * Internal function to inform all registrered objects about dragging mouse event
     * on panel.
     * @param mtx the x-coordinate of the mouse position while dragging
     * @param mty the y-coordinate of the mouse position while dragging
     */
    public void informListener(int mtx, int mty)
    {
        BresenhamPaneMouselEvent bpe = new BresenhamPaneMouselEvent (this, mtx, mty);
        Vector vtemp = (Vector)BresenhamEventListeners.clone();
        for (int x = 0; x < vtemp.size(); x++)
        {
            BresenhamEventListener target = null;
            target = (BresenhamEventListener)vtemp.elementAt(x);
            target.BresenhamEvent(bpe);
        }
    }

}
